/**
 * 应用程序入口。
 */

// general dependencies
var express = require('express');
var swagger = require("swagger-node-express");
var path = require('path');
var favicon = require('serve-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var log = require('./util/log');

// server config dependencies
var APIv1 = require('./include/endpoints').APIv1;
var PAGES = require('./include/endpoints').PAGES;

var configFile = require('./include/commons').CONFIG_FILE;
var config = require('./resources/config/' + configFile);

// routes dependencies
var index = require('./controllers/index');

var application = require('./endpoints/application.endpoint');
var users = require('./endpoints/users.endpoint');
var groups = require('./endpoints/groups.endpoint');
var chats = require('./endpoints/chats.endpoint');
var management = require('./endpoints/management.endpoint');
var push = require('./endpoints/push.endpoint');

// Application entry point
var app = express();

// Setup view engine as Jade
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'jade');

app.use(favicon(__dirname + '/public/favicon.ico', null));
app.use(logger('dev'));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({extended: false}));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));

app.use(PAGES.Home.Index, index);

app.use(APIv1.Application.Base, application);
app.use(APIv1.Chats.Base, chats);
app.use(APIv1.Users.Base, users);
app.use(APIv1.Groups.Base, groups);
app.use(APIv1.Management.Base, management);
app.use(APIv1.Push.Base, push);

/*
swagger.setAppHandler(app);
swagger.configure("http://petstore.swagger.wordnik.com", "0.1");

swagger.addValidator(
    function validate(req, path, httpMethod) {
        //  example, only allow POST for api_key="special-key"
        if ("POST" == httpMethod || "DELETE" == httpMethod || "PUT" == httpMethod) {
            var apiKey = req.headers["api_key"];
            if (!apiKey) {
                apiKey = url.parse(req.url,true).query["api_key"];
            }
            if ("special-key" == apiKey) {
                return true;
            }
            return false;
        }
        return true;
    }
);
*/

// Error handler, only handle the sync call exception
app.use(function (err, req, res, next) {
    if (err) {
        res
            .status(err.httpStatus || 500)
            .send({message: err.message});
    }

    next(err);
});

// In development mode, error handler will print stacktrace, only handle the sync call exception
if (app.get('env') === 'development') {
    app.use(function (err, req, res, next) {
        log.error(err);

        next(err);
    });
}

// Only handle the sync call exception
process.on('uncaughtException', function (err) {
    console.error(err);
});

log.info(new Date().toLocaleString() + ': Starting IM server, version ' + config.version);

module.exports = app;
